; GCB 12.8.05 make threshold adjustable

; Create a basic view of the spectrum named in the
; first argument;
; report chemical shifts when clicked.
; the spectrum is interpreted as a 2D spectrum.

(use-modules (gnome gtk)
	     (spectrum)
	     (spec-extra))

(define (confess . args) 'ok)

(if (getenv "SLEEP")
    (begin (format #t "pid --> ~a~%" (getpid))
	   (sleep 10)
	   (set! confess
		 (lambda args
		   (apply format (cons #t args))
		   (newline)))))

(define spec-name (cadr (command-line)))
(define spec
  (let ((raw-spec (spectrum-nih-from-file spec-name)))
    (if (> (spectrum-np raw-spec 2) 1)
	(spectrum-transpose raw-spec 1)
	(spectrum-nih-2d-from-file spec-name))))

(confess "spectrum loaded")

(define thres (make <gtk-adjustment> #:lower 2 #:upper 7 #:value 5.5 #:step-increment 0.1))
(define thres-spin (make <gtk-spin-button> #:digits 2 #:adjustment thres))

(define plane-spin (make <gtk-spin-button>))
(gtk-spin-button-set-range plane-spin 0 (- (spectrum-np spec 0) 1))
(gtk-spin-button-set-increments plane-spin 1 10)

(define plane-label (make <gtk-label>))

(define canv (make <hos-canvas>
	       #:width-request 300	
	       #:height-request 300))
(canvas-set-thres canv thres)
(canvas-set-draw-negative canv #t)

(confess "canvas made")

(define (goto-plane! plane)
  (canvas-set-spectrum canv
		       (spectrum-project-pt spec plane))
  (set plane-label 'label (number->string (spectrum-pt2ppm spec 0 plane))))

(if (> (spectrum-ndim spec) 2)
    (goto-plane! 0)
    (canvas-set-spectrum canv spec))
(connect plane-spin 'value-changed
	 (lambda args (goto-plane!
		       (inexact->exact
			(gtk-spin-button-get-value plane-spin))) #f))


(define shift-label (make <gtk-label>))

(let ((cursor-horiz (canvas-add-cursor-horizontal canv))
      (cursor-vert (canvas-add-cursor-vertical canv)))
  (ornaments-allow-simultaneous-grab cursor-horiz cursor-vert))

(define box (canvas-add-box canv))
(connect box 'dropped (lambda (b x1 y1 xn yn)
			(format #t "Box dropped at: ~a ~a ~a ~a~%" x1 y1 xn yn) #f))

(define main-window
  (let ((main-window (make <gtk-window>))
	(vbox (make <gtk-vbox>))
	(spec-frame (make <gtk-frame> #:label spec-name)))
    (add spec-frame canv)
    (pack-start vbox shift-label #f #f 10)
    (let ((hbox (make <gtk-hbox>)))
      (pack-start hbox thres-spin #f #f 5)
      (if (> (spectrum-ndim spec) 2)
	  (begin
	    (pack-start hbox plane-spin #f #f 5)
	    (pack-start hbox plane-label #f #f 5)))
      (pack-start vbox hbox #f #f 10))
    (pack-start vbox spec-frame #t #t 0)
    (add main-window vbox)
    main-window))

(show-all main-window)
(connect main-window 'destroy (lambda args (gtk-main-quit)))
(gtk-main)



