;; GCB 24feb06

;; CaCb chemical-shift masks:
;; use a simulated C spectrum convoluted with
;; an hncacb to filter by amino acid type.

(use-modules (gnome gtk)
	     (spectrum)
	     (spec-extra)
	     (nmr average-shift))

(define (confess . args)
  (for-each display args)
  (newline))

(format #t "pid --> ~a~%" (getpid))
; (sleep 10)

; ----- sim-spec utilities --------

(define (backing-sim-append-gaussian backing int . args)
  (let ((p (sim-peak-product-new int)))
    (define (add-profile cntr dev . args)
      (sim-peak-product-append-profile p (sim-profile-gaussian-new cntr dev))
      (if (not (null? args))
	  (apply add-profile args)))
    (apply add-profile args)
    (backing-sim-append-peak backing p)))

; ---------------------------------

(define (sim-cacb-spec ca delta-ca cb delta-cb)
  (let ((b (backing-sim-new)))
    (backing-sim-append-dimension b (dimension-sim-new 15.0 75.0 128))
    (backing-sim-append-gaussian b 1.0 ca delta-ca)
    (backing-sim-append-gaussian b -1.0 cb delta-cb)
    (backing-sim-generate-spectrum b)))

(define cacb-spec-table
  (map (lambda (aa)
	 (let ((ca-shift (shift-lookup aa 'CA))
	       (cb-shift (shift-lookup aa 'CB)))
	   (cons aa
		 (if (and ca-shift cb-shift)
		     (sim-cacb-spec (shift-mean ca-shift)
				    (shift-stddev ca-shift)
				    (shift-mean cb-shift)
				    (shift-stddev cb-shift)) #f))))
       amino-acids))

(confess "sim table done")

(define w (make <gtk-window> #:default-height 300 #:default-width 300))
(define canv (make <hos-canvas>))
(contour-set-draw-negative (painter-get-contour (canvas-get-painter canv)) #t)

;; note: for bruker, dimension order is HNC
;; transform: HNC->CHN->HCN
(define (bruker-transpose spec)  ;; HNC
  (set! spec (spectrum-transpose spec 2))  ;; CHN
  (set! spec (spectrum-transpose spec 1))  ;; HCN
  spec)

(define spec-hncacb
  (bruker-transpose (spectrum-nih-from-file "data/hncacb.DAT")))

(define hncacb-filtered-table
  (let ((spec-hncacb-chn (spectrum-transpose
			  (spectrum-nih-from-file "data/hncacb.DAT") 2)))
    (map (lambda (entry)
	   (cons (car entry)
		 (and (cdr entry)
		      (spectrum-transpose
		       (spectrum-diagonal-project
			(spectrum-convolute (cdr entry)
					    spec-hncacb-chn)) 1))))
	 cacb-spec-table)))

(confess "filtered table done")

(define spec (cdr (assoc 'SER hncacb-filtered-table)))
(define spec-nhc (spectrum-transpose spec 2))
(display cacb-spec-table)
(display hncacb-filtered-table)

(define thres (make <gtk-adjustment> #:lower 2 #:upper 7 #:value 5.5 #:step-increment 0.1))
(define thres-spin (make <gtk-spin-button> #:digits 2 #:adjustment thres))

(define plane-spin (make <gtk-spin-button>))
(gtk-spin-button-set-range plane-spin 0 (- (spectrum-np spec 0) 1))
(gtk-spin-button-set-increments plane-spin 1 10)

(define plane-label (make <gtk-label>))
(canvas-set-thres canv thres)

(define (goto-plane! plane)
  (canvas-set-spectrum canv
		       (spectrum-project-pt spec-nhc plane))
  (set plane-label 'label (number->string (spectrum-pt2ppm spec-nhc 0 plane))))

(connect plane-spin 'value-changed
	 (lambda args (goto-plane!
		       (inexact->exact
			(gtk-spin-button-get-value plane-spin))) #f))




(confess "pre goto-plane")
(goto-plane! 1)

(define shift-label (make <gtk-label>))
(define marker (canvas-add-marker canv))
(let ((refresh!
       (lambda (m x y)
	 (set shift-label 'label
	      (format #f "~a, ~a~%" x y))
	 #f)))
  (connect marker 'dropped refresh!))

(define main-window
  (let ((main-window (make <gtk-window>))
	(vbox (make <gtk-vbox>))
	(spec-frame (make <gtk-frame> #:label "HNCACB filtered")))
    (add spec-frame canv)
    (pack-start vbox shift-label #f #f 10)
    (let ((hbox (make <gtk-hbox>)))
      (pack-start hbox thres-spin #f #f 5)
      (if (> (spectrum-ndim spec) 2)
	  (begin
	    (pack-start hbox plane-spin #f #f 5)
	    (pack-start hbox plane-label #f #f 5)))
      (pack-start vbox hbox #f #f 10))
    (pack-start vbox spec-frame #t #t 0)
    (add main-window vbox)
    main-window))

(show-all main-window)
(connect main-window 'destroy (lambda args (gtk-main-quit)))
(gtk-main)

